@@ -37,16 +37,16 @@ Typical ChordPro input:
 
     {start_of_chorus}
     Swing [D]low, sweet [G]chari[D]ot,
-    Comin’ for to carry me [A7]home.
+    Comin' for to carry me [A7]home.
     Swing [D7]low, sweet [G]chari[D]ot,
-    Comin’ for to [A7]carry me [D]home.
+    Comin' for to [A7]carry me [D]home.
     {end_of_chorus}
 
     # Verse
     I [D]looked over Jordan, and [G]what did I [D]see,
-    Comin’ for to carry me [A7]home.
-    A [D]band of angels [G]comin’ after [D]me,
-    Comin’ for to [A7]carry me [D]home.
+    Comin' for to carry me [A7]home.
+    A [D]band of angels [G]comin' after [D]me,
+    Comin' for to [A7]carry me [D]home.
 
     {c: Chorus}
 
@@ -108,6 +108,12 @@ sub chordpro {
     # Establish backend.
     my $of = $options->{output};
 
+    if ( $options->{'convert-config'} ) {
+	die("\"--convert-config\" requires a single config file name\n")
+	  if @ARGV;
+	return ChordPro::Config::convert_config( $options->{'convert-config'}, $of );
+    }
+
     if ( defined($of) && $of ne "" ) {
         if ( $of =~ /\.pdf$/i ) {
             $options->{generate} ||= "PDF";
@@ -637,7 +643,20 @@ The default configuration is commented to explain its contents.
 Prints the final configuration (after processing all system, user and
 other config files)  to standard output, and exits.
 
-The final configuration is not commented. Sorry.
+=back
+
+=item B<--convert-config=>I<file>
+
+This option requires a single file name argument.
+
+The specified config file is loaded and immedeately written out in
+standard format, including schema comments. After this, the program
+exits. B<No songs are processed.>
+
+Output goes to standard output unless `--output` is used to specify an
+alternative.
+
+All other options are ignored.
 
 =back
 
@@ -811,6 +830,7 @@ sub app_setup {
 	  'print-default-config' => \$defcfg,
 	  'print-final-config'   => \$fincfg,
 	  'print-delta-config'   => \$deltacfg,
+	  'convert-config=s',
 
 	  # This aborts option scanning.
 	  'reference|R'		 => sub { $reference++; die("!FINISH!"); },
@@ -882,7 +902,6 @@ sub app_setup {
         $pod2usage->(VERBOSE => 2) if $manual;
     }
     app_ident(\*STDOUT, 0) if $version;
-    app_about(\*STDOUT, $about, 0) if $about;
 
     # If the user specified a config, it must exist.
     # Otherwise, set to a default.
@@ -943,13 +962,16 @@ sub app_setup {
     # warn(::dump($options), "\n") if $options->{debug};
 
     if ( $defcfg || $fincfg || $deltacfg ) {
-	print ChordPro::Config::config_default()
-	  if $defcfg;
-	print ChordPro::Config::config_final($deltacfg)
-	  if $fincfg || $deltacfg;
+	print ChordPro::Config::config_final( default => $defcfg,
+					      delta   => $deltacfg );
 	exit 0;
     }
 
+    if ( $about ) {
+	$::config = ChordPro::Config::configurator({});
+	app_about( \*STDOUT, $about, 0 );
+    }
+
     if ( $dump_chords ) {
 	$::config = ChordPro::Config::configurator($options);
 	require ChordPro::Chords;
@@ -984,7 +1006,10 @@ sub app_setup {
     # At this point, there should be filename argument(s)
     # unless we're embedded or just dumping chords.
     app_usage(\*STDERR, 1)
-      unless $::__EMBEDDED__ || $clo->{'dump-chords'} || @ARGV;
+      unless $::__EMBEDDED__
+      || $clo->{'dump-chords'}
+      || $clo->{'convert-config'}
+      || @ARGV;
 
     # Return result.
     $options;
@@ -992,13 +1017,27 @@ sub app_setup {
 
 sub app_ident {
     my ($fh, $exit) = @_;
-    print {$fh} ("This is ", ::runtimeinfo("short"), "\n");
+    print {$fh} ("This is ChordPro ",
+		 $::options->{reference} ? "reference " : "core ",
+		 $VERSION,
+		 $VERSION =~ /_/ ? " (Unsupported development snapshot)" : "",
+		 "\n"
+		);
     exit $exit if defined $exit;
 }
 
 sub app_about {
     my ($fh, $level, $exit) = @_;
-    print ${fh} <<EndOfAbout,
+
+    if ( $level > 2 ) {
+	require JSON::XS;
+	select $fh; $| = 1;	# otherwise no output on MacOS
+	print ${fh} ( JSON::XS->new->canonical->
+		      # pretty->
+		      utf8->convert_blessed->encode(runtime_info()) );
+    }
+    else {
+	print ${fh} <<EndOfAbout,
 
 ChordPro: A lyrics and chords formatting program.
 
@@ -1015,7 +1054,7 @@ For more information, see https://www.chordpro.org .
 Run-time information:
 EndOfAbout
       ::runtimeinfo( $level > 1 ? "extensive" : "normal" );
-
+    }
     exit $exit if defined $exit;
 }
 
@@ -1023,127 +1062,230 @@ use List::Util qw(uniq);
 
 sub ::runtimeinfo {
     my $level = shift // "normal";
+    my %i = %{runtime_info()};
+    my $fmt0   = "  %-22.22s %-10s";
+    my $fmt2   = $fmt0 . "\n";
+    my $fmt3   = $fmt0 . " (%s)\n";
+
+    my $msg;
+    for ( $i{general}->{chordpro} ) {
+	if ( $_->{aux} ) {
+	    $msg = sprintf( $fmt3,
+			    "ChordPro " . ($_->{type}//"core"), $_->{version},
+			    $_->{aux} );
+	}
+	else {
+	    $msg = sprintf( $fmt2,
+			    "ChordPro " . ($_->{type}//"core"), $_->{version} );
+	}
+    }
+    if ( $level eq "short" ) {	# used for PDF Creator info
+	$msg =~ s/^\s+//;
+	$msg =~ s/\s+/ /g;
+	$msg =~ s/\n+$//;
+	return $msg;
+    }
+
+    for ( $i{general}{perl} ) {
+	$msg .= sprintf( $fmt3, "Perl", $_->{version}, $_->{dppath} );
+    }
+
+    for ( $i{general}{packager} ) {
+	next unless defined;
+	$msg .= sprintf( $fmt2, $_->{packager}." Packager", $_->{version} );
+    }
+
+    # Determine resource path.
+    my @p;
+    my $tag = "CHORDPRO_LIB";
+    for ( @{$i{general}{library}} ) {
+	$msg .= sprintf( $fmt2, $tag, $_->{dppath} );
+	$tag = "";
+    }
+    for ( $i{general}{xdg_home} ) {
+	next unless defined;
+	$msg .= sprintf( $fmt2, "XDG_CONFIG_HOME", $_->{dppath} );
+    }
+
+    $tag = "Resource path";
+    for ( @{$i{resources}} ) {
+	$msg .= sprintf( $fmt2, $tag, $_->{dppath} );
+	$tag = "";
+    }
+
+    $tag = "FONTDIR";
+    for ( @{$i{general}{fontdir}//[]} ) {
+	next unless defined;
+	$msg .= sprintf( $fmt2, $tag, $_->{dppath} );
+	$tag = "";
+    }
+
+    for ( $i{general}{abc} ) {
+	next unless defined;
+	$msg .= sprintf( $fmt2, "ABC support", $_ );
+    }
+
+    $msg .= "\nModules and libraries:\n";
+    for ( @{$i{modules}} ) {
+	if ( $level eq "normal" ) {
+	    $msg .= sprintf( $fmt2, $_->{name}, $_->{version} );
+	}
+	else {
+	    if ( defined $_->{dppath} ) {
+		$msg .= sprintf( $fmt3, $_->{name}, $_->{version}, $_->{dppath} );
+	    }
+	    else {
+		$msg .= sprintf( $fmt2, $_->{name}, $_->{version} );
+	    }
+	}
+	if ( $_->{library} ) {
+	    if ( $_->{library} =~ /i$/ ) {
+		$msg .= sprintf( $fmt3, "  library", $_->{library}, "embedded" );
+	    }
+	    else {
+		$msg .= sprintf( $fmt2, "  library", $_->{library} );
+	    }
+	}
+    }
+
+    return $msg;
+}
+
+# Gather runtime details.
+sub runtime_info {
     my $cp = ChordPro::Paths->get;
-    my $fmt   = "  %-22.22s %-10s\n";
+    my $res;
 
     # Sometimes version numbers are localized...
     my $dd = sub { my $v = $_[0]; $v =~ s/,/./g; $v };
 
-    my $msg = sprintf( $fmt, "ChordPro core", $dd->($VERSION) );
-    $msg =~ s/core/reference/ if $::options->{reference};
-    if ( $VERSION =~ /_/ ) {
-	$msg =~ s/\n$/ (Unsupported development snapshot)\n/;
+    for ( $res->{general}{chordpro} ) {
+	$_->{version} = $dd->($VERSION);
+	$_->{type} = $::options->{reference} ? "reference" : "core";
+	$_->{aux} = "Unsupported development snapshot"
+	  if $VERSION =~ /_/;
     }
 
-    if ( $level eq "short" ) {
-	$msg =~ s/^\s+//;
-	$msg =~ s/\s+/ /g;
-	$msg =~ s/\s*\n//;
-	return $msg;
-    }
+    $res->{general}{perl} =
+      { version => "$^V",
+	path    => $^X,
+	dppath  => $cp->display($^X),
+      };
 
-    $msg .= sprintf( $fmt, "Perl", $^V );
-    $msg =~ s/\n$/sprintf(" (%s)\n", $cp->display($^X))/e;
+    $res->{general}{packager} =
+      { packager => CP->packager,
+	version  => CP->packager_version,
+      } if CP->packager;
 
-    if ( CP->packager ) {
-	$msg .= sprintf( $fmt, CP->packager." Packager", CP->packager_version );
-    }
- 
     # Determine resource path.
-    my @p;
-    my $tag = "CHORDPRO_LIB";
+    $res->{general}{library} = [];
     if ( defined $ENV{CHORDPRO_LIB} ) {
 	for ( $cp->path($ENV{CHORDPRO_LIB}) ) {
-	    $msg .= sprintf( $fmt, $tag, $cp->display($_) );
-	    $tag = "";
-	    push( @p, $cp->display($_) );
+	    push( @{$res->{general}{library}},
+		  { path   => $_,
+		    dppath => $cp->display($_) } );
 	}
     }
-    for ( qw( XDG_CONFIG_HOME ) ) {
-	if ( defined($ENV{$_}) && $ENV{$_} ne "" ) {
-	    $msg .= sprintf( $fmt, $_, $cp->display($ENV{$_}) );
+
+    if ( defined $ENV{FONTDIR} ) {
+	for ( $cp->path($ENV{FONTDIR}) ) {
+	    push( @{$res->{general}{fontdir}},
+		  { path   => $_,
+		    dppath => $cp->display($_) } );
 	}
     }
-    push( @p,  map { $cp->display($_) } @{ $cp->resdirs } );
-    $tag = "Resource path";
-    for ( uniq(@p) ) {
-	$msg .= sprintf( $fmt, $tag, $_ );
-	$tag = "";
-    }
-    eval { require ChordPro::Delegate::ABC;
-	   my $x;
-	   if ( ChordPro::Delegate::ABC::have_xs() ) {
-	       $x = ChordPro::Delegate::ABC::packaged_qjs();
-	       $msg .= sprintf( $fmt, "ABC support",
-				$x->{desc} . " (" . $x->{version} . ")" );
-	   }
-	   elsif ( $x = findexe( "abc2svg", "silent" )
-		        || findexe( "abcnode", "silent" ) ) {
-	       $msg .= sprintf( $fmt, "ABC support", $cp->display($x) );
-	   }
-	   elsif ( $x = ChordPro::Delegate::ABC::packaged_qjs() ) {
-	       $msg .= sprintf( $fmt, "ABC support",
-				$cp->display($x->{desc}) . " (" . $x->{version} . ")" );
-	   }
-    };
+
+    $res->{resources} =
+      [ map { { path => $_, dppath => $cp->display($_) } }
+	    uniq( @{ $cp->resdirs } ) ];
+
+    eval {
+	require ChordPro::Delegate::ABC;
+	my $x = ChordPro::Delegate::ABC->info();
+	$res->{general}{abc} = $x->{info} if $x->{info};
+	1;
+    } or $@ =~ /Can't locate/ or warn($@);
+
+    my @p;
 
     my $vv = sub {
 	my ( $mod ) = @_;
 	no strict 'refs';
-	$msg .= sprintf( $fmt, $mod, $dd->(${${"${mod}::"}{VERSION}}) );
-	return unless $level eq "extensive";
-	chomp($msg);
 	my $pm = $mod =~ s;::;/;gr . ".pm";
 	my $loc = $INC{$pm};
-	if ( 0 ) {
-	    $msg .= "(site)" if $loc =~ /site_perl/;
-	    $msg .= "(vendor)" if $loc =~ /vendor_perl/;
-	    $msg .= "(ChordPro)" if $loc =~ /ChordPro\/lib/;
-	    $msg .= "(private)" if $loc =~ /$ENV{HOME}\/lib\/perl5/;
-	}
-	else {
-	    $msg .= "($1)" if $loc =~ /^(.*)\/\Q$pm\E$/;
-	}
-	$msg .= "\n";
+	push( @p, { name    => $mod,
+		    version => "".$dd->(${${"${mod}::"}{VERSION}}),
+		    path    => $loc,
+		    dppath  => $loc =~ /^(.*)\/\Q$pm\E$/ ? $cp->display($1) : $loc,
+		  } );
     };
-    $msg .= "\nModules and libraries:\n";
+
     if ( defined $Wx::VERSION ) {
 	no strict 'subs';
-	$msg .= sprintf( $fmt, "wxPerl", $dd->($Wx::VERSION) );
-	$msg .= sprintf( $fmt, "wxWidgets", $dd->(Wx::wxVERSION) );
+	push( @p,
+	      { name => "wxPerl",    version => $dd->($Wx::VERSION)  },
+	      { name => "wxWidgets", version => $dd->(Wx::wxVERSION) } );
     }
 
     local $SIG{__WARN__} = sub {};
     local $SIG{__DIE__} = sub {};
     $vv->("Storable");
     $vv->("Object::Pad");
-    eval { require Text::Layout;
+    eval {
+	require Text::Layout;
 	$vv->("Text::Layout");
     };
-    eval { require HarfBuzz::Shaper;
+    eval {
+	require HarfBuzz::Shaper;
 	$vv->("HarfBuzz::Shaper");
-	$msg .= sprintf( $fmt, "HarfBuzz library", $dd->(HarfBuzz::Shaper::hb_version_string()) );
+	$p[-1]->{library} = $dd->(HarfBuzz::Shaper::hb_version_string());
     };
     $vv->("File::LoadLines");
-    eval { require PDF::Builder;
+    eval {
+	require PDF::Builder;
 	$vv->("PDF::Builder");
     };
-    eval { require PDF::API2;
+    eval {
+	require PDF::API2;
 	$vv->("PDF::API2");
-	eval { require PDF::API2::XS;
-	       $vv->("PDF::API2::XS");
+	eval {
+	    require PDF::API2::XS;
+	    $vv->("PDF::API2::XS");
 	};
     };
-    eval { require SVGPDF;
+    eval {
+	require SVGPDF;
 	$vv->("SVGPDF");
     };
-    eval { require Font::TTF;
+    eval {
+	require Font::TTF;
 	$vv->("Font::TTF");
     };
-    eval { require JavaScript::QuickJS;
+    eval {
+	require JavaScript::QuickJS;
 	$vv->("JavaScript::QuickJS");
     };
-    return $msg;
+    my $i = json_parser();
+    $vv->( $i->{parser} );
+    $p[-1]->{relaxed} = "relaxed" if $i->{relaxed};
+
+    eval {
+	require JSON::XS;
+	$vv->("JSON::XS");
+    };
+
+    $res->{modules} = [ @p ];
+
+    $res->{metadata} = $::config->{metadata}{keys};
+
+    @p = ( qw(title subtitle),
+	   ( grep { !/^(sub)?title$/ } sort(@{$::config->{metadata}{keys}}) ),
+	   grep { !/^(sub)?title$/ } (keys(%{ChordPro::Song::_directives()})) );
+    $res->{directives} = [ @p ];
+
+    $res->{directive_abbrevs} = ChordPro::Song::_directive_abbrevs();
+
+    return $res;
 }
 
 sub splitpath {
@@ -1161,6 +1303,8 @@ sub app_usage {
     for ( qw( config userconfig sysconfig) ) {
 	$cfg{$_} = $configs{$_} || "no default";
     }
+
+    ####TODO: weed out for --reference.
     print ${fh} <<EndOfUsage;
 Usage: $0 [ options ] [ file ... ]
 
@@ -1250,9 +1394,7 @@ dir, and the contents of environment variable C<FONTDIR>. In any case,
 the filename should point to a valid TrueType (C<.ttf>) or OpenType
 (C<.otf>) font.
 
-If it is not a filename, it must be the name one of the built-in fonts.
-
-Built-in 'Adobe Core Fonts':
+If it is not a filename, it must be the name one of the built-in PDF core fonts:
 
   Courier                             Symbol
   Courier-Bold                        Times-Bold
@@ -1263,17 +1405,6 @@ Built-in 'Adobe Core Fonts':
   Helvetica-BoldOblique
   Helvetica-Oblique
 
-Built-in 'Windows Fonts':
-
-  Georgia                             Webdings
-  Georgia,Bold                        Wingdings
-  Georgia,BoldItalic
-  Georgia,Italic
-  Verdana
-  Verdana,Bold
-  Verdana,BoldItalic
-  Verdana,Italic
-
 =head1 MOTIVATION
 
 Why a rewrite of Chordii?
