@@ -288,7 +288,7 @@ sub generate_song ( $s ) {
 	    if ( $elt->{chords} ) {
 		$text = "";
 		for ( 0..$#{ $elt->{chords} } ) {
-		    $text .= "[" . $elt->{chords}->[$_] . "]"
+		    $text .= "[" . fq(chord( $s, $elt->{chords}->[$_])) . "]"
 		      if $elt->{chords}->[$_] ne "";
 		    $text .= $elt->{phrases}->[$_];
 		}
@@ -346,6 +346,14 @@ sub generate_song ( $s ) {
 	    next;
 	}
 
+	if ( $elt->{type} eq "control" ) {
+	    if ( $elt->{name} =~ /^(\w+)-(size|color|font)/ ) {
+		my $t = "{$1$2: " . $elt->{value} . "}";
+		push( @s, $t ) unless $t =~ s/^\{\Kchorus/text/r eq $s[-1];
+	    }
+	    next;
+	}
+
 	if ( $elt->{type} eq "ignore" ) {
 	    push( @s, $elt->{text} );
 	    next;
