@@ -56,7 +56,7 @@ my $memorizing;			# if memorizing (a.o.t. recalling)
 # Keep track of unknown chords, to avoid dup warnings.
 my %warned_chords;
 
-my $re_chords;			# for chords
+our $re_chords;			# for chords
 my $intervals;			# number of note intervals
 my @labels;			# labels used
 
@@ -170,6 +170,7 @@ sub parse_song {
 		next unless -s $cf;
 		warn("Config[song]: $cf\n") if $options->{verbose};
 		my $have = ChordPro::Config::get_config($cf);
+		ChordPro::Config::config_expand_font_shortcuts ( $have );
 		push( @configs, ChordPro::Config::prep_configs( $have, $cf) );
 		last;
 	    }
@@ -217,9 +218,7 @@ sub parse_song {
     $config->unlock;
 
     if ( %$defs ) {
-	my $c = $config->hmerge( prp2cfg( $defs, $config ) );
-	bless $c => ref($config);
-	$config = $c;
+	prpadd2cfg( $config, %$defs );
     }
 
     for ( qw( transpose transcode decapo lyrics-only ) ) {
@@ -331,10 +330,15 @@ sub parse_song {
 	    $_ .= $cont;
 	}
 
+	# Uncomment this to allow \uDXXX\uDYYY (surrogate) escapes.
+	s/ \\u(d[89ab][[:xdigit:]]{2})\\u(d[cdef][[:xdigit:]]{2})
+	 / pack('U*', 0x10000 + (hex($1) - 0xD800) * 0x400 + (hex($2) - 0xDC00) )
+	   /igex;
+
 	# Uncomment this to allow \uXXXX escapes.
 	s/\\u([0-9a-f]{4})/chr(hex("0x$1"))/ige;
 	# Uncomment this to allow \u{XX...} escapes.
-	# s/\\u\{([0-9a-f]+)\}/chr(hex("0x$1"))/ige;
+	s/\\u\{([0-9a-f]+)\}/chr(hex("0x$1"))/ige;
 
 	$diag->{orig} = $_;
 	# Get rid of TABs.
@@ -346,9 +350,9 @@ sub parse_song {
 
 	for my $pp ( "all", "env-$in_context" ) {
 	    if ( $prep->{$pp} ) {
-		0&&warn("PRE:  ", $_, "\n");
+		$config->{debug}->{pp} && warn("PRE:  ", $_, "\n");
 		$prep->{$pp}->($_);
-		0&&warn("POST: ", $_, "\n");
+		$config->{debug}->{pp} && warn("POST: ", $_, "\n");
 		if ( /\n/ ) {
 		    my @a = split( /\n/, $_ );
 		    $_ = shift(@a);
@@ -556,6 +560,10 @@ sub parse_song {
 			}
 		    }
 		    $opts = $a->{opts} = { %$opts, %{$a->{opts}} };
+		    if ( $opts->{align} && $opts->{x} && $opts->{x} =~ /\%$/ ) {
+			do_warn( "Useless combination of x percentage with align (align ignored)" );
+			delete $opts->{align};
+    }
 
 		    my $def = !!$id;
 		    $id //= "_Image".$assetid++;
@@ -618,9 +626,9 @@ sub parse_song {
 	if ( /^\s*\{(.*)\}\s*$/ ) {
 	    my $dir = $1;
 	    if ( $prep->{directive} ) {
-		# warn("PRE:  ", $_, "\n");
+		$config->{debug}->{pp} && warn("PRE:  ", $_, "\n");
 		$prep->{directive}->($dir);
-		# warn("POST: ", $_, "\n");
+		$config->{debug}->{pp} && warn("POST: {", $dir, "}\n");
 	    }
 	    $self->add( type => "ignore",
 			text => $_ )
@@ -648,9 +656,9 @@ sub parse_song {
 
 	if ( /\S/ ) {
 	    if ( $prep->{songline} ) {
-		# warn("PRE:  ", $_, "\n");
+		$config->{debug}->{pp} && warn("PRE:  ", $_, "\n");
 		$prep->{songline}->($_);
-		# warn("POST: ", $_, "\n");
+		$config->{debug}->{pp} && warn("POST: ", $_, "\n");
 	    }
 	    if ( $config->{settings}->{flowtext}
 		 && @{ $self->{body}//[] } ) {
@@ -747,8 +755,8 @@ sub parse_song {
     }
 
     if ( $config->{diagrams}->{sorted} ) {
-	@used_chords =
-	  sort ChordPro::Chords::chordcompare @used_chords;
+	sub byname { ChordPro::Chords::chordcompare($a,$b) }
+	@used_chords = sort byname @used_chords;
     }
 
     # For headings, footers, table of contents, ...
@@ -972,6 +980,7 @@ sub decompose_grid {
     $line =~ s/^\s+//;
     $line =~ s/\s+$//;
     return ( tokens => [] ) if $line eq "";
+    local $re_chords = qr/(\[.*?\])/;
 
     my $orig;
     my %res;
@@ -1117,10 +1126,12 @@ my %abbrevs = (
    cb	      => "comment_box",
    cf	      => "chordfont",
    ci	      => "comment_italic",
+   col	      => "colums",
    colb	      => "column_break",
    cs	      => "chordsize",
    eob	      => "end_of_bridge",
    eoc	      => "end_of_chorus",
+   eog	      => "end_of_grid",
    eot	      => "end_of_tab",
    eov	      => "end_of_verse",
    g	      => "diagrams",
@@ -1130,6 +1141,7 @@ my %abbrevs = (
    ns	      => "new_song",
    sob	      => "start_of_bridge",
    soc	      => "start_of_chorus",
+   sog	      => "start_of_grid",
    sot	      => "start_of_tab",
    sov	      => "start_of_verse",
    st	      => "subtitle",
@@ -1138,6 +1150,10 @@ my %abbrevs = (
    ts         => "textsize",
 	      );
 
+# Use by: runtimeinfo.
+sub _directives { \%directives }
+sub _directive_abbrevs { \%abbrevs }
+
 my $dirpat;
 
 sub parse_directive {
@@ -1240,17 +1256,23 @@ sub directive {
 	@chorus = (), $chorus_xpose = $chorus_xpose_dir = 0
 	  if $in_context eq "chorus";
 	if ( $in_context eq "grid" ) {
-	    if ( $arg eq "" ) {
+	    my $kv;
+	    my $shape = $arg;
+	    if ( $arg =~ /\w+="/ ) {
+		$kv = parse_kv($arg);
+		$shape = $kv->{shape};
+	    }
+	    if ( $shape eq "" ) {
 		$self->add( type => "set",
 			    name => "gridparams",
 			    value => $grid_arg );
 	    }
-	    elsif ( $arg =~ m/^
+	    elsif ( $shape =~ m/^
 			      (?: (\d+) \+)?
 			      (\d+) (?: x (\d+) )?
 			      (?:\+ (\d+) )?
 			      (?:[:\s+] (.*)? )? $/x ) {
-		do_warn("Invalid grid params: $arg (must be non-zero)"), return
+		do_warn("Invalid grid params: $shape (must be non-zero)"), return
 		  unless $2;
 		$grid_arg = [ $2, $3//1, $1//0, $4//0 ];
 		$self->add( type => "set",
@@ -1258,11 +1280,17 @@ sub directive {
 			    value =>  [ @$grid_arg, $5||"" ] );
 		push( @labels, $5 ) if length($5||"");
 	    }
-	    elsif ( $arg ne "" ) {
+	    elsif ( $shape ne "" ) {
 		$self->add( type => "set",
 			    name => "gridparams",
-			    value =>  [ @$grid_arg, $arg ] );
-		push( @labels, $arg );
+			    value =>  [ @$grid_arg, $shape ] );
+		push( @labels, $shape );
+	    }
+	    if ( ($kv->{label}//"") ne "" ) {
+		$self->add( type  => "set",
+			    name  => "label",
+			    value => $kv->{label} );
+		push( @labels, $kv->{label} );
 	    }
 	    $grid_cells = [ $grid_arg->[0] * $grid_arg->[1],
 			    $grid_arg->[2],  $grid_arg->[3] ];
@@ -1276,7 +1304,7 @@ sub directive {
 		  $xpose + ($config->{settings}->{transpose}//0 );
 	    }
 	    my $kv = {};
-	    if ( $arg =~ /\b(id|label|scale|split|spread|width|align|center)=(.+)/ ) {
+	    if ( $arg =~ /\w+=["'](.+)/ ) {
 		$kv = parse_kv($arg);
 	    }
 	    else {
@@ -1297,7 +1325,7 @@ sub directive {
 	    if ( $arg =~ /^label=/ ) {
 		$arg = parse_kv($arg)->{label};
 	    }
-	    elsif ( $arg =~ /\b(id|scale|split|spread|width|align|center)=(.+)/ ) {
+	    elsif ( $arg =~ /\w+=["'](.+)/ ) {
 		# Doesn't look like a label. Assume a mistake.
 		do_warn("Garbage in start_of_$in_context: $arg (ignored)\n");
 		$arg = "";
@@ -1373,49 +1401,10 @@ sub directive {
 		    value => $arg,
 		  );
 
-	# THIS IS BASICALLY A COPY OF THE CODE IN Config.pm.
-	# TODO: GENERALIZE.
-	my $ccfg = {};
-	my @k = split( /[:.]/, $1 );
-	my $c = \$ccfg;		# new
-	my $o = $config;	# current
-	my $lk = pop(@k);	# last key
+	$config->unlock;
+	prpadd2cfg( $config, $1 => $arg );
+	$config->lock;
 
-	# Step through the keys.
-	foreach ( @k ) {
-	    $c = \($$c->{$_});
-	    $o = $o->{$_};
-	}
-
-	# Turn hash.array into hash.array.> (append).
-	if ( ref($o) eq 'HASH' && ref($o->{$lk}) eq 'ARRAY' ) {
-	    $c = \($$c->{$lk});
-	    $o = $o->{$lk};
-	    $lk = '>';
-	}
-
-	# Final key. Merge array if so.
-	if ( ( $lk =~ /^\d+$/ || $lk eq '>' || $lk eq '<' )
-	       && ref($o) eq 'ARRAY' ) {
-	    unless ( ref($$c) eq 'ARRAY' ) {
-		# Only copy orig values the first time.
-		$$c->[$_] = $o->[$_] for 0..scalar(@{$o})-1;
-	    }
-	    if ( $lk eq '>' ) {
-		push( @{$$c}, $arg );
-	    }
-	    elsif ( $lk eq '<' ) {
-		unshift( @{$$c}, $arg );
-	    }
-	    else {
-		$$c->[$lk] = $arg;
-	    }
-	}
-	else {
-	    $$c->{$lk} = $arg;
-	}
-
-	$config->augment($ccfg);
 	upd_config();
 
 	return 1;
@@ -1439,18 +1428,24 @@ sub dir_chorus {
     my $chorus = @chorus ? dclone(\@chorus) : [];
 
     if ( @$chorus && $arg && $arg ne "" ) {
+	my $label = $arg;
+	my $kv;
+	if ( $arg =~ /\w+="/ ) {
+	    $kv = parse_kv($arg);
+	    $label = $kv->{label};
+	}
 	if ( $chorus->[0]->{type} eq "set" && $chorus->[0]->{name} eq "label" ) {
-	    $chorus->[0]->{value} = $arg;
+	    $chorus->[0]->{value} = $label;
 	}
 	else {
 	    unshift( @$chorus,
 		     { type => "set",
 		       name => "label",
-		       value => $arg,
+		       value => $label,
 		       context => "chorus",
 		     } );
 	}
-	push( @labels, $arg )
+	push( @labels, $label )
 	  if $config->{settings}->{choruslabels};
     }
 
@@ -1544,12 +1539,13 @@ sub dir_image {
 	    $opts{lc($k)} = $v;
 	}
 	elsif ( $k =~ /^(x|y)$/i
-		&& $v =~ /^([-+]?\d+(?:\.\d+)?\%?)$/ ) {
+		&& $v =~ /^(?:base[+-])?([-+]?\d+(?:\.\d+)?\%?)$/ ) {
 	    $opts{lc($k)} = $v;
 	}
 	elsif ( $k =~ /^(scale)$/
-		&& $v =~ /^(\d+(?:\.\d+)?)(%)?$/ ) {
-	    $opts{lc($k)} = $2 ? $1/100 : $1;
+		&& $v =~ /^(\d+(?:\.\d+)?)(%)?(?:,(\d+(?:\.\d+)?)(%)?)?$/ ) {
+	    $opts{lc($k)} = [ $2 ? $1/100 : $1 ];
+	    $opts{lc($k)}->[1] = $3 ? $4 ? $3/100 : $3 : $opts{lc($k)}->[0];
 	}
 	elsif ( $k =~ /^(center|border|spread|persist)$/i ) {
 	    if ( $k eq "center" ) {
@@ -1571,7 +1567,7 @@ sub dir_image {
 	elsif ( $k =~ /^(type)$/i && $v ne "" ) {
 	    $opts{type} = $v;
 	}
-	elsif ( $k =~ /^(label)$/i && $v ne "" ) {
+	elsif ( $k =~ /^(label|href)$/i && $v ne "" ) {
 	    $opts{lc($k)} = $v;
 	}
 	elsif ( $k =~ /^(anchor)$/i
@@ -1596,6 +1592,10 @@ sub dir_image {
 	do_warn( "Missing image source\n" );
 	return;
     }
+    if ( $opts{align} && $opts{x} && $opts{x} =~ /\%$/ ) {
+	do_warn( "Useless combination of x percentage with align (align ignored)" );
+	delete $opts{align};
+    }
 
     # If the image uri does not have a directory, look it up
     # next to the song, and then in the images folder of the
@@ -1629,12 +1629,17 @@ sub dir_image {
     # Store as asset.
     if ( $uri ) {
 	my $opts;
-	$opts->{type} = $opts{type}    if $opts{type};
-	$opts->{persist} = $opts{persist} if $opts{persist};
-	delete $opts{$_} for qw( type persist );
+	for ( qw( type persist href ) ) {
+	    $opts->{$_} = $opts{$_} if defined $opts{$_};
+	    delete $opts{$_};
+	}
+	for ( qw( spread ) ) {
+	    $opts->{$_} = $opts{$_} if defined $opts{$_};
+	}
 
 	if ( $id && %opts ) {
-	    do_warn("Asset definition \"$id\" does not take attributes");
+	    do_warn("Asset definition \"$id\" does not take attributes",
+		   " (" . join(" ",sort keys %opts) . ")");
 	    return;
 	}
 
@@ -1707,6 +1712,9 @@ sub dir_meta {
 	    @vals = map { s/s\+$//; $_ }
 	      split( quotemeta($config->{metadata}->{separator}), $vals[0] );
 	}
+	else {
+	    pop(@vals) if $vals[0] eq '';
+	}
 	my $m = $self->{meta};
 
 	# User and instrument cannot be set here.
@@ -1726,6 +1734,7 @@ sub dir_meta {
 		    local( $self->{chordsinfo}->{_dummy_} ) = { root_ord => 0 };
 		    $self->parse_chord($val);
 		};
+		do_warn("Illegal key: \"$val\"\n"), next unless $info;
 		my $name = $info->name;
 		my $act = $name;
 
@@ -2064,7 +2073,7 @@ sub define_chord {
 	# frets N N ... N
 	elsif ( $a eq "frets" ) {
 	    my @f;
-	    while ( @a && $a[0] =~ /^(?:[0-9]+|[-xXN])$/ && @f < $strings ) {
+	    while ( @a && $a[0] =~ /^(?:-?[0-9]+|[-xXN])$/ && @f < $strings ) {
 		push( @f, shift(@a) );
 	    }
 	    if ( @f == $strings ) {
